#include <limits.h>

#include <windows.h>
#include <ime.h>

#include "resource.h"
#include "main.h"


void DrawLineNumber( HWND hWnd )
{
	TCHAR       szBuf[16];
	int         nTopLine;
	int         nMaxLine;
	HDC         hDC;
	HFONT       hFont;
	HFONT       hFontOld;
	COLORREF	clrBgOld;
	COLORREF	clrFgOld;
	HBRUSH		hBgBrush;
	RECT        rect;
	TEXTMETRIC  tm;


	hFont = (HFONT)SendMessage( hWnd, WM_GETFONT, 0, 0 );

	nTopLine = SendMessage( hWnd, EM_GETFIRSTVISIBLELINE, 0, 0 );
	nMaxLine = SendMessage( hWnd, EM_GETLINECOUNT       , 0, 0 );
	SendMessage( hWnd, EM_GETRECT, 0, (LPARAM)&rect );
	
	hDC = GetDC( hWnd );
	clrBgOld = GetBkColor( hDC );
	clrFgOld = GetTextColor( hDC );
	hBgBrush = (HBRUSH)SendMessage( GetParent(hWnd), WM_CTLCOLOREDIT, (WPARAM)hDC, (LPARAM)hWnd );
	hFontOld = SelectObject( hDC, hFont );
	GetTextMetrics( hDC, &tm );
	rect.bottom = rect.bottom / tm.tmHeight * tm.tmHeight;
	while ( rect.top < rect.bottom && nTopLine < nMaxLine ) {
		wsprintf( szBuf, "%05d : ", nTopLine + 1 );
		TextOut( hDC, 2, rect.top, szBuf, 8 );
		rect.top += tm.tmHeight;
		nTopLine++;
	}
	if ( hFontOld ) {
		SelectObject( hDC, hFontOld );
	}
	if ( hBgBrush ) {
		SetBkColor( hDC, clrBgOld );
		SetTextColor( hDC, clrFgOld );
	}
	ReleaseDC( hWnd, hDC );
}

void RecalcRect( HWND hWnd )
{
	RECT       rect;
	HDC        hDC;
	HFONT      hFont;
	HFONT      hFontOld;
	SIZE       size;


	hFont = (HFONT)SendMessage( hWnd, WM_GETFONT, 0, 0 );

	hDC = GetDC( hWnd );
	hFontOld = SelectObject( hDC, hFont );
	GetTextExtentPoint32( hDC, "00000 : ", 8, &size );
	if ( hFontOld ) {
		SelectObject( hDC, hFontOld );
	}
	ReleaseDC( hWnd, hDC );

	SendMessage( hWnd, EM_GETRECT, 0, (LPARAM)&rect );
	rect.left = rect.left + size.cx;
	SendMessage( hWnd, EM_SETRECTNP, 0, (LPARAM)&rect );
}

LRESULT CALLBACK EditViewProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	static AC *ac;
	WNDPROC	lpfnOrgProg = (WNDPROC)GetWindowLong( hWnd, GWL_USERDATA );
	LRESULT lRes;

	switch ( uMsg ) {
	case WM_SETFONT:
		lRes = CallWindowProc( lpfnOrgProg, hWnd, uMsg, wParam, lParam );
		RecalcRect( hWnd );
		return lRes;

	case WM_PAINT:
		lRes = CallWindowProc( lpfnOrgProg, hWnd, uMsg, wParam, lParam );
		DrawLineNumber( hWnd );
		return lRes;

	case WM_SIZE:
		lRes = CallWindowProc( lpfnOrgProg, hWnd, uMsg, wParam, lParam );
		RecalcRect( hWnd );
		return lRes;

	case WM_KEYDOWN:
//		{
//			TCHAR szDebug[10];
//
//			wsprintf( szDebug, "0x%02X\n", wParam );
//			OutputDebugString( szDebug );
//		}
		lRes = CallWindowProc( lpfnOrgProg, hWnd, uMsg, wParam, lParam );
		switch ( wParam ) {
		case VK_RETURN:
		case VK_DELETE:
		case 0x08:	// BS
			DrawLineNumber( hWnd );
			break;

		case VK_CAPITAL:
		case VK_KANA:
		case VK_NUMLOCK:
			SendMessage( GetParent(hWnd), uMsg, wParam, lParam );
			break;
		}
		InvalidateRect( hWnd, NULL, FALSE );
		return lRes;

	default:
		return CallWindowProc( lpfnOrgProg, hWnd, uMsg, wParam, lParam );
	}
	return 0;
}

HWND CreateEditView( AC *ac )
{
	HWND	      hWnd;
	WNDPROC	      lpfnOrgProg;
	OSVERSIONINFO ovi;

	hWnd = CreateWindowEx( WS_EX_CLIENTEDGE
//						 , "RICHEDIT", ""
						 , "EDIT", ""
						 , WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL
						 | ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_LEFT | ES_OEMCONVERT
						 | ES_MULTILINE | ES_WANTRETURN | ES_NOHIDESEL
						 , 0, 0, CW_USEDEFAULT, CW_USEDEFAULT
						 , ac->hWndMain
						 , (HMENU)IDC_MAINEDIT
						 , ac->hInst
						 , ac
						 );

	ZeroMemory( &ovi, sizeof(OSVERSIONINFO) );
	ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if ( GetVersionEx(&ovi) && ovi.dwPlatformId == VER_PLATFORM_WIN32_NT ) {
		SendMessage( hWnd, EM_LIMITTEXT, 0, 0 );		// for WindowsNT
	} else {
		// "64422"́AJł̎lB
		SendMessage( hWnd, EM_LIMITTEXT, 64422, 0 );	// for Windows95
	}
	UpdateWindow( hWnd );

	lpfnOrgProg = (WNDPROC)GetWindowLong( hWnd, GWL_WNDPROC );
	SetWindowLong( hWnd, GWL_USERDATA, (LONG)lpfnOrgProg );
	SetWindowLong( hWnd, GWL_WNDPROC , (LONG)EditViewProc );

	return hWnd;
}