#include <stdio.h>
#include <windows.h>


static BOOL VsOpenPlofile( HKEY *lphKey, LPCTSTR lpszSection )
{
	static TCHAR  szSection[BUFSIZ];
	       DWORD  dwDisposition;
	       LONG   lRetCode;


	if ( *lphKey != NULL ) {
		return FALSE;
	}

	if ( lpszSection ) {
		wsprintf( szSection, "SOFTWARE\\GORRY\\SPICE\\%s", lpszSection );
	} else {
		strcpy( szSection, "SOFTWARE\\GORRY\\SPICE" );
	}

	lRetCode = RegCreateKeyEx( HKEY_CURRENT_USER
							 , szSection
							 , 0
							 , NULL
							 , REG_OPTION_NON_VOLATILE
							 , KEY_ALL_ACCESS
							 , NULL
							 , lphKey
							 , &dwDisposition
							 );

	if ( lRetCode != ERROR_SUCCESS ) {
		*lphKey = NULL;
		return FALSE;
	}

//	if ( dwDisposition == REG_CREATED_NEW_KEY ) {
//		return FALSE;
//	}
	return TRUE;
}

void VsClosePlofile( HKEY *lphKey )
{
	if ( *lphKey != NULL ) {
		RegCloseKey( *lphKey );
		*lphKey = NULL;
	}
}

LPCTSTR VsGetProfileString( LPCTSTR lpszSection, LPCTSTR lpszKey, LPCTSTR lpszDefault )
{
	static TCHAR  szBuf[BUFSIZ];
	       LONG   lRetCode;
	       DWORD  dwType = REG_SZ;
	       DWORD  dwLen  = BUFSIZ;
	       HKEY   hKey   = NULL;


	if ( VsOpenPlofile( &hKey, lpszSection ) == FALSE ) {
		return lpszDefault;
	}
	
	lRetCode = RegQueryValueEx(	hKey
							  , lpszKey
							  , NULL
							  , &dwType
							  , (LPBYTE)szBuf
							  , &dwLen
							  );
	VsClosePlofile( &hKey );

	if ( lRetCode != ERROR_SUCCESS ) {
		return lpszDefault;
	}

	return szBuf;
}

BOOL VsPutProfileString( LPCTSTR lpszSection, LPCTSTR lpszKey, LPCTSTR lpszValue )
{
	LONG   lRetCode;
	HKEY   hKey    = NULL;


	if ( VsOpenPlofile( &hKey, lpszSection ) == FALSE ) {
		return FALSE;
	}
	
	lRetCode = RegSetValueEx( hKey
							, lpszKey
							, 0
							, REG_SZ
							, (LPBYTE)lpszValue
							, lstrlen(lpszValue)
							);
	VsClosePlofile( &hKey );

	if ( lRetCode != ERROR_SUCCESS ) {
		return FALSE;
	}

	return TRUE;
}

int VsGetProfileInt( LPCTSTR lpszSection, LPCTSTR lpszKey, int nDefault )
{
	LONG   lRetCode;
	DWORD  dwValue;
	DWORD  dwType = REG_DWORD;
	DWORD  dwLen  = sizeof(DWORD);
	HKEY   hKey   = NULL;

	if ( VsOpenPlofile( &hKey, lpszSection ) == FALSE ) {
		return nDefault;
	}
	
	lRetCode = RegQueryValueEx(	hKey
							  , lpszKey
							  , NULL
							  , &dwType
							  , (LPBYTE)&dwValue
							  , &dwLen
							  );
	VsClosePlofile( &hKey );

	if ( lRetCode != ERROR_SUCCESS ) {
		return nDefault;
	}

	return (int)dwValue;
}

BOOL VsPutProfileInt( LPCTSTR lpszSection, LPCTSTR lpszKey, int nValue )
{
	LONG   lRetCode;
	DWORD  dwValue = (DWORD)nValue;
	HKEY   hKey    = NULL;


	if ( VsOpenPlofile( &hKey, lpszSection ) == FALSE ) {
		return FALSE;
	}
	
	lRetCode = RegSetValueEx( hKey
							, lpszKey
							, 0
							, REG_DWORD
							, (LPBYTE)&dwValue
							, sizeof(DWORD)
							);
	VsClosePlofile( &hKey );

	if ( lRetCode != ERROR_SUCCESS ) {
		return FALSE;
	}

	return TRUE;
}

BOOL VsGetProfileFont( LPCTSTR lpszSection, LPCTSTR lpszKey, LOGFONT *lplf )
{
	LONG   lRetCode;
	DWORD  dwType = REG_BINARY;
	DWORD  dwLen  = sizeof(LOGFONT);
	HKEY   hKey   = NULL;


	if ( VsOpenPlofile( &hKey, lpszSection ) == FALSE ) {
		return FALSE;
	}
	
	lRetCode = RegQueryValueEx(	hKey
							  , lpszKey
							  , NULL
							  , &dwType
							  , (LPBYTE)lplf
							  , &dwLen
							  );
	VsClosePlofile( &hKey );

	if ( lRetCode != ERROR_SUCCESS ) {
		return FALSE;
	}

	return TRUE;
}

BOOL VsPutProfileFont( LPCTSTR lpszSection, LPCTSTR lpszKey, LOGFONT *lplf )
{
	LONG   lRetCode;
	HKEY   hKey    = NULL;


	if ( VsOpenPlofile( &hKey, lpszSection ) == FALSE ) {
		return FALSE;
	}
	
	lRetCode = RegSetValueEx( hKey
							, lpszKey
							, 0
							, REG_BINARY
							, (LPBYTE)lplf
							, sizeof(LOGFONT)
							);
	VsClosePlofile( &hKey );

	if ( lRetCode != ERROR_SUCCESS ) {
		return FALSE;
	}

	return TRUE;
}
