#include <windows.h>
#include <commctrl.h>

#include "main.h"
#include "resource.h"


LPCTSTR MakeOption( BOOL bClock, BOOL bCheck, BOOL bLoop, int nLoop, BOOL bFade, int nFade )
{
	static TCHAR szOption[256];


	if ( bLoop ) {
		if ( bFade ) {
			wsprintf( szOption, " -L%d -F%d ", nLoop, nFade );
		} else {
			wsprintf( szOption, " -L%d ", nLoop );
		}
	} else {
		lstrcpy( szOption, " " );
	}
	if ( bClock ) {
		lstrcat( szOption, "-PC " );
	}

	if ( bCheck ) {
		lstrcat( szOption, "-PS " );
	}

	return szOption;
}

int GetLoop( HWND hDlg )
{
	BOOL bRes;
	int  nRes;

	nRes = (int)GetDlgItemInt( hDlg, IDC_LOOP, &bRes, FALSE );
	if ( !bRes ) {
		MessageBeep( MB_OK );
		SetFocus( GetDlgItem(hDlg, IDC_LOOP) );
		return -1;
	}
	if ( nRes < 1 || nRes > 127 ) {
		MessageBeep( MB_OK );
		SetFocus( GetDlgItem(hDlg, IDC_LOOP) );
		return -1;
	}
	return nRes;
}

int GetFade( HWND hDlg )
{
	BOOL bRes;
	int  nRes;

	nRes = (int)GetDlgItemInt( hDlg, IDC_FADE, &bRes, FALSE );
	if ( !bRes ) {
		MessageBeep( MB_OK );
		SetFocus( GetDlgItem(hDlg, IDC_FADE) );
		return -1;
	}
	if ( nRes < 1 || nRes > 16384 ) {
		MessageBeep( MB_OK );
		SetFocus( GetDlgItem(hDlg, IDC_FADE) );
		return -1;
	}
	return nRes;
}

BOOL CALLBACK SetupDlgProc( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	static AC	*ac;
	static BOOL	 bClock;
	static BOOL	 bCheck;
	static BOOL	 bLoop;
	static BOOL	 bFade;
	static int   nLoop;
	static int   nFade;


	switch ( uMsg ) {
	case WM_INITDIALOG:
		ac = (AC *)lParam;

		bClock = ac->bClock;
		bCheck = ac->bCheck;
		bLoop  = ac->bLoop;
		bFade  = ac->bFade;
		nLoop  = ac->nLoop;
		nFade  = ac->nFade;

		SendDlgItemMessage( hDlg, IDC_LOOPSPIN, UDM_SETRANGE, 0, MAKELPARAM(  127, 1) );
		SendDlgItemMessage( hDlg, IDC_FADESPIN, UDM_SETRANGE, 0, MAKELPARAM(16384, 1) );

		CheckDlgButton( hDlg, IDC_CLOCK    , bClock );
		CheckDlgButton( hDlg, IDC_CHECKSUM , bCheck );
		CheckDlgButton( hDlg, IDC_NOENDLOOP, bLoop  );
		CheckDlgButton( hDlg, IDC_FADEOUT  , bFade  );
		SetDlgItemInt( hDlg, IDC_LOOP, ac->nLoop, FALSE );
		SetDlgItemInt( hDlg, IDC_FADE, ac->nFade, FALSE );
		EnableWindow( GetDlgItem(hDlg, IDC_LOOP   ), bLoop );
		EnableWindow( GetDlgItem(hDlg, IDC_FADEOUT), bLoop );
		if ( ac->bLoop ) {
			EnableWindow( GetDlgItem(hDlg, IDC_FADE), bFade );
		} else {
			EnableWindow( GetDlgItem(hDlg, IDC_FADE), bLoop );
		}
		SetDlgItemText( hDlg
					  , IDC_PREVIEW
					  , MakeOption( bClock, bCheck, bLoop, nLoop, bFade, nFade )
					  );
		return TRUE;

	case WM_COMMAND:
		if ( HIWORD(wParam) == EN_CHANGE ) {
			switch ( LOWORD(wParam) ) {
			case IDC_LOOP:
				nLoop = GetLoop( hDlg );
				if ( nLoop < 0 ) {
					return TRUE;
				}
				break;

			case IDC_FADE:
				nFade = GetFade( hDlg );
				if ( nFade < 0 ) {
					return TRUE;
				}
				break;
			}

			if ( LOWORD(wParam) != IDC_PREVIEW ) {
				SetDlgItemText( hDlg
							  , IDC_PREVIEW
							  , MakeOption( bClock, bCheck, bLoop, nLoop, bFade, nFade )
							  );
			}
			return TRUE;
		}
		switch ( LOWORD(wParam) ) {
		case IDOK:
			if ( bLoop ) {
				nLoop = GetLoop( hDlg );
				if ( nLoop < 0 ) {
					return TRUE;
				}
				if ( bFade ) {
					nFade = GetFade( hDlg );
					if ( nFade < 0 ) {
						return TRUE;
					}
				}
			}
			ac->nLoop  = nLoop;
			ac->nFade  = nFade;
			ac->bClock = bClock;
			ac->bCheck = bCheck;
			ac->bLoop  = bLoop;
			ac->bFade  = bFade;
			VsPutProfileInt( "Local\\VisualSpice\\SMC", "CLOCK", ac->bClock );
			VsPutProfileInt( "Local\\VisualSpice\\SMC", "CHECK", ac->bCheck );
			VsPutProfileInt( "Local\\VisualSpice\\SMC", "LOOP" , ac->bLoop  );
			VsPutProfileInt( "Local\\VisualSpice\\SMC", "FADE" , ac->bFade  );
			VsPutProfileInt( "Local\\VisualSpice\\SMC", "LOOPCOUNT", ac->nLoop  );
			VsPutProfileInt( "Local\\VisualSpice\\SMC", "FADESPEED", ac->nFade  );
			EndDialog( hDlg, IDOK );
			return TRUE;

		case IDCANCEL:
			EndDialog( hDlg, IDCANCEL );
			return TRUE;

		case IDC_CLOCK:
			bClock = SendDlgItemMessage( hDlg
									   , IDC_CLOCK
									   , BM_GETCHECK
									   , 0
									   , 0
									   );
			SetDlgItemText( hDlg
						  , IDC_PREVIEW
						  , MakeOption( bClock, bCheck, bLoop, nLoop, bFade, nFade )
						  );
			break;

		case IDC_CHECKSUM:
			bCheck = SendDlgItemMessage( hDlg
									   , IDC_CHECKSUM
									   , BM_GETCHECK
									   , 0
									   , 0
									   );
			SetDlgItemText( hDlg
						  , IDC_PREVIEW
						  , MakeOption( bClock, bCheck, bLoop, nLoop, bFade, nFade )
						  );
			break;

		case IDC_NOENDLOOP:
			bLoop = SendDlgItemMessage( hDlg
									  , IDC_NOENDLOOP
									  , BM_GETCHECK
									  , 0
									  , 0
									  );
			EnableWindow( GetDlgItem(hDlg, IDC_LOOP   ), bLoop );
			EnableWindow( GetDlgItem(hDlg, IDC_FADEOUT), bLoop );
			if ( bLoop ) {
				EnableWindow( GetDlgItem(hDlg, IDC_FADE), bFade );
			} else {
				EnableWindow( GetDlgItem(hDlg, IDC_FADE), bLoop );
			}
			SetDlgItemText( hDlg
						  , IDC_PREVIEW
						  , MakeOption( bClock, bCheck, bLoop, nLoop, bFade, nFade )
						  );
			break;

		case IDC_FADEOUT:
			bFade = SendDlgItemMessage( hDlg
									  , IDC_FADEOUT
									  , BM_GETCHECK
									  , 0
									  , 0
									  );
			EnableWindow( GetDlgItem(hDlg, IDC_FADE), bFade );
			SetDlgItemText( hDlg
						  , IDC_PREVIEW
						  , MakeOption( bClock, bCheck, bLoop, nLoop, bFade, nFade )
						  );
			break;
		}
	}
	return FALSE;
}

void CompileOption( AC *ac )
{
	DialogBoxParam( ac->hInst, MAKEINTRESOURCE(IDD_SETUP), ac->hWndMain, SetupDlgProc, (LPARAM)ac );
}