#include <windows.h>

#include "Splitter.h"

#define GWL_SPLIT_STYLE	(sizeof(LONG) * 0)
#define GWL_SPLIT_SIZE	(sizeof(LONG) * 1)
#define GWL_SPLIT_POS	(sizeof(LONG) * 2)
#define GWL_SPLIT_MAX	(sizeof(LONG) * 3)
#define GWL_SPLIT_MIN	(sizeof(LONG) * 4)


BOOL GetSplitterStyle( HWND hWnd )
{
	return (BOOL)GetWindowLong( hWnd, GWL_SPLIT_STYLE );
}

BOOL SetSplitterStyle( HWND hWnd, BOOL bStyle )
{
	return (BOOL)SetWindowLong( hWnd, GWL_SPLIT_STYLE, (LONG)bStyle );
}

int GetSplitterSize( HWND hWnd )
{
	return (int)GetWindowLong( hWnd, GWL_SPLIT_SIZE );
}

int SetSplitterSize( HWND hWnd, int nSize )
{
	return (int)SetWindowLong( hWnd, GWL_SPLIT_SIZE, (LONG)nSize );
}

int GetSplitterMax( HWND hWnd )
{
	return (int)GetWindowLong( hWnd, GWL_SPLIT_MAX );
}

int SetSplitterMax( HWND hWnd, int nVal )
{
	return (int)SetWindowLong( hWnd, GWL_SPLIT_MAX, (LONG)nVal );
}

int GetSplitterMin( HWND hWnd )
{
	return (int)GetWindowLong( hWnd, GWL_SPLIT_MIN );
}

int SetSplitterMin( HWND hWnd, int nVal )
{
	return (int)SetWindowLong( hWnd, GWL_SPLIT_MIN, (LONG)nVal );
}

int GetSplitterPos( HWND hWnd )
{
	return (int)GetWindowLong( hWnd, GWL_SPLIT_POS );
}

int SetSplitterPos( HWND hWnd, int nPos, BOOL bUpdate )
{
	int		nTop;
	int		nBottom;
	int		nLeft;
	int		nRight;
	RECT	rectParent;

	GetClientRect( GetParent(hWnd), &rectParent );
	if ( GetSplitterStyle(hWnd) ) {
		nLeft   = nPos;
		nRight  = GetSplitterSize( hWnd );
		nTop    = rectParent.top;
		nBottom = rectParent.bottom;
	} else {
		nLeft   = rectParent.left;
		nRight  = rectParent.right;
		nTop    = nPos;
		nBottom = GetSplitterSize( hWnd );
	}
	MoveWindow( hWnd, nLeft, nTop, nRight, nBottom, bUpdate );

	return (int)SetWindowLong( hWnd, GWL_SPLIT_POS, (LONG)nPos );
}

LRESULT CALLBACK SplitterProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

	switch ( uMsg ) {
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;

				SetSplitterStyle( hWnd, (BOOL)(lpcs->lpCreateParams) );
				if ( GetSplitterStyle(hWnd) ) {
					SetSplitterSize( hWnd, GetSystemMetrics(SM_CXSIZEFRAME) );
				} else {
					SetSplitterSize( hWnd, GetSystemMetrics(SM_CYSIZEFRAME) );
				}
				SetSplitterMin( hWnd, (int)0     );
				SetSplitterMax( hWnd, (int)65535 );
			}
			break;

		case WM_SIZE:
			{
				int nPos = GetSplitterPos( hWnd );

				SetSplitterPos( hWnd, nPos, TRUE );
			}
			break;

		case WM_LBUTTONDOWN:
			SetCapture( hWnd );
			break;

		case WM_LBUTTONUP:
			ReleaseCapture();
			break;

		case WM_MOUSEMOVE:
			if ( wParam == MK_LBUTTON ) {
				POINT point;
				RECT  rect;
				int   nPos;
				int   nMax;
				int   nMin;
				HWND  hWndParent = GetParent( hWnd );

				GetCursorPos( &point );
				ScreenToClient( hWndParent, &point );

				if ( GetSplitterStyle(hWnd) ) {
					nPos = point.x;
				}else {
					nPos = point.y;
				}
				nMin = GetSplitterMin( hWnd );
				nMax = GetSplitterMax( hWnd );
				if ( nPos >= nMin && nPos <= nMax ) {
					SetSplitterPos( hWnd, nPos, TRUE );
					{
						TCHAR szDBG[20];

						wsprintf( szDBG, "%d\n", nPos );
						OutputDebugString( szDBG );
					}
					GetClientRect( hWndParent, &rect );
					SendMessage( hWndParent
							   , WM_SIZE
							   , SIZE_RESTORED
							   , MAKELPARAM( rect.right, rect.bottom )
							   );
				}
			}
			break;

		case WM_SETRANGE:
			SetSplitterMin( hWnd, (int)wParam );
			SetSplitterMax( hWnd, (int)lParam );
			break;

		default:
			return DefWindowProc( hWnd, uMsg, wParam, lParam );
	}
	return 0;
}

HWND CreateSplitter( HWND hWndParent, BOOL bStyle, UINT wID, HCURSOR hCursor, HINSTANCE hInst, int nPos )
{
	static LPTSTR lpszClassName = "";

	HWND	hWnd;


	if ( strlen(lpszClassName) == 0 ) {
		WNDCLASS wcThisClass;

		strcpy( lpszClassName, "SPLITTER" );

		wcThisClass.style         = 0;//CS_HREDRAW | CS_VREDRAW;
		wcThisClass.lpfnWndProc   = SplitterProc;
		wcThisClass.cbClsExtra    = 0;
		wcThisClass.cbWndExtra    = sizeof(LONG) * 5;
		wcThisClass.hInstance     = hInst;
		wcThisClass.hIcon         = NULL;
		wcThisClass.hCursor       = hCursor;
		wcThisClass.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
		wcThisClass.lpszMenuName  = NULL;
		wcThisClass.lpszClassName = lpszClassName;

		RegisterClass( &wcThisClass );
	}

	hWnd = CreateWindow( lpszClassName
					   , ""
					   , WS_CHILD | WS_VISIBLE
					   , 0
					   , 0
					   , 0
					   , 0
					   , hWndParent
					   , (HMENU)wID
					   , hInst
					   , (LPVOID)bStyle
					   );

	SetSplitterPos( hWnd, nPos, TRUE );

	return hWnd;
}